/* XND_RCRY.C     this file read all data for crystal phases */

#include "xnd_def.h"
#include "xnd_rdh.h"

/* ------------------------------------------------------------------ */

static int check_atom (P_(void))      /*  internal number of atom in bond */
{   char nom[10];
	ATOM *atom;
	int i,j,k;
	fscanf (coord_fich," %s",nom);
	fprintf (list_fich," %6s",nom);
	for (i=0,atom=l_phase->atom; i<l_phase->n_atom; i++,atom++)
		if (strcmp(nom, atom->nom)==0) return (i);
	for (k=0;k<l_phase->nb_bloc;k++)
	for (j=0,atom=l_phase->bl_ortho[k]; j<l_phase->bl_atom[k]; 
		j++,i++,atom++)
		if (strcmp(nom, atom->nom)==0) return (i);
	termine (10,"unknown atom");
	return (0);
	}

/* ------------------------------------------------------------------ */

static void check_dif (P_(char *)espece_lue)
#ifdef NO_PROTO
char *espece_lue;
#endif
{    char espece[11];
	char *p_esp;
	l_atom->xdif = l_atom->ndif = l_atom->mdif = l_atom->uxdif = l_atom->undif = NULL;
        l_atom->adif = NULL;
	if (lect->nature & LECT_NAT_RX)
	{   strcpy (espece,"0");
	    strcat (espece,espece_lue);
	    p_esp = espece+1;
	    while (isdigit(*p_esp)) p_esp++;
	    l_atom->xdif = get_dif(p_esp,1);
	    if (lect->nature & LECT_NAT_RXA) l_atom->adif = get_adif(p_esp);
	    if (l_phase->uncoherent & DIF_COMPTON)
	    {    p_esp--;
	        *p_esp='@';
		l_atom->uxdif = get_dif(p_esp,1);
		}
	    }
	if (lect->nature & LECT_NAT_N)
	{       strcpy (espece,"0");
	        strcat (espece,espece_lue);
		p_esp = espece+1;
		while (isalpha(*p_esp)) p_esp++;
		*p_esp=0;
		p_esp = isalpha(espece[1]) ? espece : espece+1;
		l_atom->ndif = get_dif(p_esp,1);
		if (lect->nature & LECT_NAT_NM)
	        espece[0]='_';
		p_esp = espece;
		l_atom->mdif = get_dif(p_esp,0);
		if (l_phase->uncoherent & DIF_COMPTON)
	        {   espece[0]='$';
		    l_atom->undif = get_dif(p_esp,1);
		}
	    }	
	}

/* ------------------------------------------------------------------ */

static DIST *fget_dist (P_(int) angle, P_(int) nb)
#ifdef NO_PROTO
int angle, nb;
#endif
{   DIST  *p_dist,*p_0;
	COORD *coord;
	char   nom[NAME_LG];
	int i,j,n;
	if (nb==0) return (NULL);
	l_atom = l_phase->atom;
	p_0 = p_dist =  (DIST *) tst_alloc(nb, sizeof(DIST), "dist");
	for (i=0;i<nb;i++,p_dist++)
	{     p_dist->remark  = read_remark ();
              fscanf (coord_fich,"%d %f", &p_dist->nb_coord, &p_dist->sigma);
	      fprintf (list_fich,"\n %d liaisons,  sigma = %.3f\n",
			p_dist->nb_coord, p_dist->sigma);
	      fget_var (&p_dist->dist,1,0,VAR_STD,angle==1 ? T_DIST : T_ANGLE,
		       (void *) txt_dist);
		coord = p_dist->coord = (COORD *) tst_alloc(p_dist->nb_coord,
			sizeof(COORD),"coord");
		for (j=0;j<p_dist->nb_coord;j++,coord++)
		{   fprintf(list_fich," #%2d ",j);
		    coord->at0 = check_atom ();
		    coord->endat = (ENDAT*) tst_alloc(angle,
			sizeof(ENDAT),"coord");
		    for (n=0;n<angle;n++)
		    {   fprintf (list_fich," -> ");
			coord->endat[n].at = check_atom ();
			fscanf (coord_fich," %[^;\n]%*[;\n]", nom);
			while (!analyse_xyz_pos(nom, coord->endat[n].coef, l_phase->symgrp));
			fprint_xyz_pos (list_fich, coord->endat[n].coef,
				 (n == angle-1) ? '\n' : ' ');
			}
		    }
		n_pen += p_dist->nb_coord ;
		}
	return (p_0);
	}

/* ------------------------------------------------------------------ */
ATOM *fget_atom (P_(int) nombre, P_(int) terme)
#ifdef NO_PROTO
int nombre, terme;
#endif
{   ATOM   *ret_atom;
    int    j,cas;
    char   espece[10];
	 ret_atom = l_atom = (ATOM *) tst_alloc(nombre,sizeof(ATOM),"atoms");
    for (j=0;j<nombre;j++,l_atom++)
    {   l_atom->remark = read_remark();
	fscanf (coord_fich," %s %s",l_atom->nom,espece);
	fprintf (list_fich," %s %s",l_atom->nom,espece);
	if (terme != -1)
	{   fscanf (coord_fich,"%d",&cas);
	    fprintf (list_fich," %d:(", cas);
	    l_atom->nature = cas;
		 tst_abort ((cas>NATURE_MAX),"Unknown atomic nature ");
	    l_atom->nb = COORD_0;
	    if (cas & B_ANISO) 
            {    l_atom->nb += COORD_ANIS;
	         fprintf (list_fich," B_ANISO ");
                 }
	    else fprintf (list_fich," B_ISO ");
	    if (cas & MAGNET)
	    {    l_atom->nb += COORD_MAGN;
		 lect->n_magnet++;
	         fprintf (list_fich," MAGNET ");
		 }
	    l_atom->n_modul = ( cas & MODUL )>>2;
	    if ( l_atom->n_modul )
	    {    lect->n_modul++;
	         fprintf (list_fich," MODULATED=%d ", l_atom->n_modul);
		 }
	    fprintf (list_fich,")");
		 if (terme)  tst_abort (((cas & MAGNET) != 0 || l_atom->n_modul != 0 ),
		 "magnetic or modulated atoms are not allowed in rigid body");
	    }
	else l_atom->nb = 1;
	check_dif (espece);
	fget_var (&l_atom->x,l_atom->nb, 0, VAR_ATOM, T_ATOM, (void *)NULL);
	l_phase->nb_par += l_atom->nb;
	if ( l_atom->n_modul )
	{    fget_var (&l_atom->modul, l_atom->n_modul*AMODUL, AMODUL, VAR_ORIEN, T_MODUL,(void *)txt_modul);
	     l_phase->nb_par += l_atom->n_modul * AMODUL;
	     }
	}
    return (ret_atom);
	 }
/* ------------------------------------------------------------------ */
void fget_phase_prof(P_(void))
{    int i,j;

     push_err_msg("fget_phase_prof");
     if (l_phase->n_prof)
     {   l_phase->remark_prof = read_remark();
	 l_phase->c_prof = ( int *) tst_alloc (l_phase->n_prof,sizeof(int),"profil");
	 for (j=0;j<l_phase->n_prof;j++)
	 {  fscanf(coord_fich,"%d",&l_phase->c_prof[j]);
	    p_orien(j,1);
	    }
	 fprintf(list_fich,"\n");
	 fget_var (&l_phase->prof,PROF*l_phase->n_prof,PROF, VAR_ORIEN,
                       T_PROFIL, (void *)txt_prof);
	 VAR_init(&l_phase->prof);
	 VAR_io_id
	   for (i=0;i<l_phase->n_prof;i++) for(j=0;j<4;j++)
	       if (var_param(i*PROF+j)) phase_prof_par++;
	 fprintf(list_fich,"\n");
	 fget_var  (&l_phase->assym,max_assym*PASSYM*l_phase->n_prof,PASSYM, VAR_ORIEN,
		       T_PASSYM, (void *)&txt_passym[0]);
	 VAR_init(&l_phase->assym);
	 VAR_io_id
	   for (i=0;i<l_phase->n_prof;i++) for(j=0;j<max_assym*PASSYM;j++)
	       if (var_param(i*max_assym*PASSYM+j)) phase_prof_par++;
	 }
      pop_err_msg();
      }
/* ------------------------------------------------------------------ */
void fget_phase_orien(P_(void))
{   int j;
	 if (l_phase->nt_orien)
    {   l_phase->remark_orien = read_remark();
	l_phase->c_orien = ( int *) tst_alloc (l_phase->nt_orien, sizeof(int),"orien");
	for (j=0;j<l_phase->nt_orien;j++)
	{   fscanf(coord_fich,"%d",&l_phase->c_orien[j]);
	    p_orien(j,0);
	    }
	fget_var  (&l_phase->orien,ORIEN*l_phase->nt_orien,ORIEN, VAR_ORIEN,
		       T_ORIEN, (void *)&txt_orien[0]);
	    }
     }
/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */
TMODUL *fget_magnet (P_(int) cas)
#ifdef NO_PROTO
int cas;
#endif
{  char  nom[NAME_LG];
   char   *msg;
   TMODUL  *magnet;
   SATEL *satel;
   int i, j, dim, nb_sat;

   push_err_msg("fget_magnet");
   magnet = (TMODUL *) tst_alloc (1, sizeof(TMODUL), "modulation");

   fscanf (coord_fich," %d%d%d", &magnet->dimension,
	 &magnet->nature, &magnet->nb_sat);
   dim = magnet->dimension;
   nb_sat = magnet->nb_sat;

   fprintf (list_fich,
   "\n %s : dimension %d, nature %d (%scommensurable), satellites a lire %d\n",
	 cas==0?"MAGNETISME":"MODULATION",
	 magnet->dimension, magnet->nature, magnet->nature>=1?"":"in",
	 magnet->nb_sat);
   tst_abort ((dim > MAX_DIM_MODUL)," size not allowed");
   if (magnet->nature > 0)
   {     fscanf (coord_fich," %f", &magnet->t_orig);
        fprintf (list_fich,
	    "superstructure order %d, section %f\n", magnet->nature, magnet->t_orig);
	}
   else
   if (magnet->nature == 0)
   fprintf (list_fich,
   "Using default (%d points) in calculating integrals for incomensurate\n",
    NB_INCOM);
   else 
   fprintf (list_fich,
   "Using %d points in calculating integrals for incomensurate\n",
     -magnet->nature);

    if (cas == 0 ) nb_sat++;
   tot_phase += nb_sat - 1;
   fscanf (coord_fich," %s",nom);
	fprintf(list_fich," symetry group : '%s'\n", nom);
   magnet->symgrp = get_sym (nom);
   tst_abort ( (magnet->symgrp->holoedre != dim),
	  " Size of complentary group for modulation does not agree with modulation size");
   tst_abort ( (magnet->symgrp->nops != l_phase->symgrp->nops),
	  " NOps of complentary group for modulation does not agree with basic group");

   satel = magnet->satel = (SATEL *) tst_alloc (magnet->nb_sat,
      sizeof(SATEL), "satellite");

  fprintf (list_fich,"\n les familles de raies :\n");
  for (i=0; i<nb_sat; i++, satel++)
   {   msg = (i==0 && cas==0) ?
	       "Structure normale" : "indice(s) complementaire(s) :";
       fscanf (coord_fich," %d", &satel->nb_hkl);
       fprintf (list_fich," nb_hkl %d (%s) %s",
	  satel->nb_hkl, satel->nb_hkl==0?"genere":"lu", msg);
       if(i!=0 || cas!=0) for (j=0; j<dim;j++)
       {   fscanf(coord_fich,"%d",&satel->hkl_sat[j]);
	   fprintf (list_fich,"  %d ", satel->hkl_sat[j]);
	   }
       else for (j=0; j<dim;j++) satel->hkl_sat[j] = 0;
       fprintf (list_fich," -> SATELLITES #%d\n", i);
       }
    fprintf (list_fich,"\n");

   fget_var (&magnet->vecteur, magnet->dimension * VECTEUR, VECTEUR,
	      VAR_STD, "Vecteur", (void *)txt_vecteur );

   pop_err_msg();
   return magnet;
   }
/* ------------------------------------------------------------------ */
void fget_magnet_hkl (P_(TMODUL *) magnet)
#ifdef NO_PROTO
TMODUL * magnet;
#endif
{
    SATEL *satel;
    int i, nb_sat;
    nb_sat = magnet->nb_sat;
    satel = magnet->satel;

    for (i=0; i<nb_sat; i++, satel++)
    {   if (satel->nb_hkl > 0)
	    fget_var (&satel->intens, satel->nb_hkl,
	    magnet->dimension, VAR_HKL,
			  "Int.", (void *) &satel->hkl);
       }
    }
/* ------------------------------------------------------------------ */
void fget_crystal (P_(void))
{
   float  *p_bond;
	int    i,j,nb0;
	char   nom[NAME_LG];

	push_err_msg("fget_crystal");
	fscanf (coord_fich," %s",nom);
	fprintf(list_fich," Symmetry group : '%s'\n", nom);
	l_phase->symgrp = get_sym (nom);
	fscanf (coord_fich,"%d%d%d%d%d%d",&l_phase->n_orien, &l_phase->n_prof,
		 &l_phase->n_atom, &l_phase->nb_bloc, 
		 &l_phase->uncoherent, &l_phase->nb_bond);
	fprintf (list_fich,"  %d Polynome(s) en orientation, %d en profil,\n\
  %d atom(s), %d rigid body,\n  %d uncoherent code,  %d T restraints\n\n",
		l_phase->n_orien, l_phase->n_prof, l_phase->n_atom,
		l_phase->nb_bloc, l_phase->uncoherent,l_phase->nb_bond);
  if (l_phase->n_orien > 0 && n_exper > 1)
	  fprintf (list_fich," n_orien > 0 : prefered orientation function is common to all experiment!\n");
  if (l_phase->n_orien < 0 && n_exper > 1)
	  fprintf (list_fich," n_orien < 0 : prefered orientation function depend of the experiment!\n %d * %d functions have to be read\n",
		  -l_phase->n_orien, n_exper);
      if (l_phase->n_orien >= 0)
      {   l_phase->nt_orien = l_phase->n_orien;
          }
      else
      { l_phase->n_orien *=-1;
        l_phase->nt_orien = l_phase->n_orien * n_exper;
        }
	if ( l_phase->uncoherent )
		 fprintf (list_fich," uncoherent code implies : %s%s%s\n",
			l_phase->uncoherent & DIF_COMPTON ? " compton":"",
	      l_phase->uncoherent & DIF_TDS_ISO ? " iso tds":"",
	      l_phase->uncoherent & DIF_TDS_PIC ? " pic tds":"");
	      
	fflush (list_fich);
	fget_var (&l_phase->cell,CELL,0, VAR_STD,"CELL", (void *)txt_cell);

	if (l_phase->nature == PHASE_MAGNET) l_phase->magnet = fget_magnet(0);
	else l_phase->magnet =  NULL;
	if (l_phase->nature == PHASE_MODUL) l_phase->modul  = fget_magnet(1);
	else  l_phase->modul =  NULL;

	fget_phase_orien();
	fget_phase_prof();
				
	fflush (list_fich);
	nb0 = n_par;
	lect->n_magnet = lect->n_modul  = 0;
	l_phase->atom = fget_atom (l_phase->n_atom, 0);
	l_phase->nb_atom = l_phase->n_atom;
	if (l_phase->nb_bloc != 0)
	{   fget_var (&l_phase->bloc, BCOOR*l_phase->nb_bloc,BCOOR, VAR_ORIEN,
		    T_BLOC, (void *)txt_bloc);
	    l_phase->bl_atom = (int *) tst_alloc (l_phase->nb_bloc,
		sizeof(int),"bloc");
	    l_phase->bl_ortho = (ATOM **) tst_alloc (l_phase->nb_bloc,
		sizeof(ATOM *),"bloc");
	    l_phase->bl_relat = (ATOM **) tst_alloc (l_phase->nb_bloc,
		sizeof(ATOM *),"bloc");
	    for (j=0;j<l_phase->nb_bloc;j++)
	    {  fscanf(coord_fich,"%d",&l_phase->bl_atom[j]);
	       fprintf(list_fich," Le bloc #%d_ contient %d atomes\n",
			j,l_phase->bl_atom[j]);
	       tst_abort ((l_phase->bl_atom[j]==0),"BLOC vide");
	       l_phase->nb_atom += l_phase->bl_atom[j];
	       l_phase->bl_ortho[j] = fget_atom (l_phase->bl_atom[j], 1);
	    }   }
	if (n_par == nb0) l_phase->nb_par = 0;

	if (l_phase->nature != PHASE_MAGNET && lect->n_magnet) 
                  fprintf (list_fich,
	 " magnetic atoms are skipped: phase is not known as magnetic!\n");
	if (l_phase->nature != PHASE_MODUL && lect->n_modul) 
                  fprintf (list_fich,
	 " modulated atoms are skipped: phase is not known as modulated!\n");

	l_phase->intens.ordre = 0;
	switch (l_phase->nature)
	{   case PHASE_GENERE : 
	        if ( l_phase->nb_atom == 0 ) fprintf(list_fich,
	            " There is ZERO atom in the cell, assuming Fhkl = 1 for all lines\n");
	        break;
            case PHASE_HKL_LU :
                fget_var (&l_phase->intens,l_phase->nb_hkl, 0, VAR_HKL,
			  "Int.", NULL);
	        if ( l_phase->nb_atom == 0 && l_phase->intens.ordre == 0) fprintf(list_fich,
	            " There is ZERO atom in the cell, assuming Fhkl = 1 for all lines\n");
                break;
             case PHASE_MAGNET : fget_magnet_hkl (l_phase->magnet);
                break;
             case PHASE_MODUL  : fget_magnet_hkl (l_phase->modul);
                break;
                }
	if (l_phase->uncoherent)
	{   fget_var (&l_phase->dif_ech,1, 0, VAR_STD, "TdS.", NULL);
	    if (l_phase->uncoherent & DIF_TDS_PIC )
	    {   fscanf (coord_fich,"%d",&l_phase->nb_tds);
                fprintf( list_fich, " %d pic tds\n",l_phase->nb_tds);
   	        fget_var (&l_phase->tds,l_phase->nb_tds*DIF, DIF, VAR_HKL, "TdS.", (void *)&l_phase->hkl_tds);
	        }
	    }    
	tst_abort(((l_phase->n_atom==0) &&
		(l_phase->nb_bond || l_phase->nb_angl)),
		 " Usage contrainte invalide");
	if (l_phase->nb_bond)
	{	l_phase->remark_bond = read_remark();
		p_bond = l_phase->bond = (float *) tst_alloc(l_phase->nb_bond,
			sizeof(FLOAT),"bond");
		fscanf (coord_fich,"%d%d",&l_phase->nb_dist,&l_phase->nb_angl);
		fprintf (list_fich,"\n %d distance(s) %d angle(s)\n",
			l_phase->nb_dist,l_phase->nb_angl);
		for (i=0;i<l_phase->nb_bond;i++,p_bond++)
		{	fscanf (coord_fich,"%f",p_bond);
			fprintf (list_fich,"%3d T_bond %f\n",i,*p_bond);
			}
		l_phase->dist = fget_dist (1, l_phase->nb_dist);
		l_phase->angl = fget_dist (2, l_phase->nb_angl);
		}

	pop_err_msg();
	}
/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */


