/* XND_RSYM.C    lecture des symetries */

#include "xnd_def.h"
#include "xnd_rdh.h"
#include <string.h>
/* ------------------------------------------------------------------ */

void fprint_xyz_pos (P_(FILE *) file, P_(float *) l_coef, P_(char) end_char)
#ifdef NO_PROTO
FILE *file;
float *l_coef;
char end_char;
#endif
{   int   i,j;
    float val;
    float *coef;
    int   num,deno,first;
    int    dim = 3;
    int    dim2;

    push_err_msg("fprint_xyz_pos(xnd_rsym.c)");
    if (l_symgrp->systeme =='*') dim =l_symgrp->holoedre;
    dim2 = 1 + dim;

    fprintf(file,"  ");
    for (j=0,coef=l_coef; j<dim; j++,coef++)
    {   first =1;
	for (i=0; i<dim2; i++)
	{   val = coef[dim*i];
	    if (fabs(val)>EPSILON)
	    {   if (val>0) num = floor (val * 48 + 0.5);
	        else num = ceil (val* 48 - 0.5);
		deno = 48;
		while ((num%2==0) && (deno%2==0)) { num/=2; deno/=2; };
		while ((num%3==0) && (deno%3==0)) { num/=3; deno/=3; };
		if (i==dim) fprintf (file,"%+d",num);
		else if (num==1) fprintf (file,"%s%c",first?"":"+",'X'+i);
		else if (num==-1) fprintf (file,"-%c",'X'+i);
		else fprintf (file,"%d%c",num,'X'+i);
		if (deno!=1) fprintf (file,"/%d",deno);
		first = 0;
		};
	     };
	if (j!=dim-1) fprintf (file,", ");
	};
    fprintf (file,"%c",end_char);
    pop_err_msg();
    }

/* ------------------------------------------------------------------ */

void print_sym(P_(SYMGRP *) symgrp, P_(int) cas)
#ifdef NO_PROTO
SYMGRP *symgrp;
int cas;
#endif
{   int i;
    float *l_coef;
    char *txt;

    push_err_msg("print_sym(xnd_rsym.c)");
    if (symgrp->systeme != '*')
    {     switch (symgrp->systeme)
          {    case 'C' : txt = "Cubique"; break;
               case 'Q' : txt = "Quadratique"; break;
               case 'H' : txt = "Hexagonal"; break;
               case 'R' : txt = "Rhomboedrique/Hexagonal"; break;
               case 'T' : txt = "Trigonal"; break;
               case 'O' : txt = "Orthorhombique"; break;
               case 'M' : txt = "MonocliniqueB"; break;
               case 'N' : txt = "MonocliniqueC"; break;
               case 'L' : txt = "MonocliniqueA"; break;
               case 'A' : txt = "Anortic=triclinique"; break;
                };
	if (cas) fprintf(list_fich," Symetry group '%s'", symgrp->nom);
	fprintf(list_fich, 
            "  %s system %c (%s) holoedric %d, %d %s, %d operations \n",
		symgrp->comment, symgrp->systeme,txt,symgrp->holoedre, 
                symgrp->centro, symgrp->centro==1?"(-x,-y,-z) * ":"NotCentered",
                symgrp->nops);
        }
     else
     {
        if (cas) fprintf(list_fich,
         " Complement de symetrie pour structure modulee '%s'", symgrp->nom);
	fprintf(list_fich, 
            "  %s %c dimension %d, (%d) %d operations \n",
		symgrp->comment, symgrp->systeme, 
                symgrp->holoedre, symgrp->centro, symgrp->nops);
           }
        
        l_symgrp = symgrp;
	for (i=0,l_coef=symgrp->coef; i<symgrp->nops; i++,l_coef+=symgrp->size)
        {   fprintf (list_fich," #%2d ", i);
	    fprint_xyz_pos (list_fich, l_coef, '\n');
	    }
        fprintf(list_fich,"\n");
    pop_err_msg();
    }

/* ------------------------------------------------------------------ */

void new_sym(P_(SYMGRP *) symgrp)
#ifdef NO_PROTO
SYMGRP *symgrp;
#endif
{   int i;
    char c;
    float *l_coef;

   push_err_msg("new_sym(xnd_rsym.c)");
   fprintf(new_fich," %s %s %c %d %d %d\n",
		symgrp->nom, symgrp->comment, symgrp->systeme,
                symgrp->holoedre, symgrp->centro,symgrp->nops);
    l_symgrp = symgrp;
    for (i=0,l_coef=symgrp->coef;i<symgrp->nops;i++,l_coef+=symgrp->size)
	{   c = (i%4)!=3 ? ';' : '\n';
	    fprint_xyz_pos ( new_fich, l_coef, c);
		}
    fprintf(new_fich,"\n\n");
    pop_err_msg();
    }

/* ------------------------------------------------------------------ */

int analyse_xyz_pos(P_(char *) ligne, P_(float *) l_coef, P_(SYMGRP *)l_sym )
#ifdef NO_PROTO
char *ligne;
float *l_coef;
SYMGRP *l_sym;
#endif
{   float  num,deno;
    int    div,dig,sig;
    int    j, range;
    char   *ligne_p,car;
    int    dim = 3;
    int    dim2;
    int    valid =0;
    
    push_err_msg("analyse_xyz_pos(xnd_rsym.c)");
    if (l_sym->systeme =='*') dim = l_sym->holoedre;
        dim2 = dim*dim;
        if (strlen(ligne)>=NAME_LG) ligne[NAME_LG-1]=0;
	strup (ligne);
	ligne_p=ligne;
	num = deno = 0;
	j = dig = div = 0;
	sig = 1;
	range = -1;
	while ((car= *ligne_p)!=0)
	{   ligne_p++;
		if (isdigit(car))
		{   dig++;
			if (div==0) num = num*10+car-'0';
			else deno = deno*10+car-'0';
			valid++;
			}
	     else if (car=='/')
	     {   if (num==0) num = 1;
			 div=1;
			 }
	     else switch (car)
		 {   case 'X' :
			 case 'Y' :
			 case 'Z' : range = j + dim *(car-'X');
				    if (num==0) num = 1;
			            valid++;
				     break;
			 case ',' :
			 case '-' :
			 case '+' : if (div!=0)  num /= deno;
					if (range == -1) range = j + dim2;
					if (num!=0) l_coef [range] = sig*num;
					range = -1;
					num=deno=0; div=0;
					sig = car=='-'?-1:1;
					if (car==',') j++;
					break;
			 default  : if (car !=' ' && car != '\r')
		{  if (l_sym!=NULL) fprintf(list_fich," Groupe '%s'\n",l_sym->nom);
		   fprintf(list_fich,
		    " Erreur interpretation de '%c' (%02x) dans '%s'\n", 
		        car, car, ligne);
		    };
			 };
		};
	if (div!=0)  num /= deno;
	if (range == -1) range = j + dim2;
	if (num!=0) l_coef [range] = sig*num;
	pop_err_msg();
	if (valid < dim) return 0;
	return 1;
	}

/* ------------------------------------------------------------------ */

SYMGRP *read_sym (P_(FILE *) file)
#ifdef NO_PROTO
FILE *file;
#endif
{   char  name[31], ligne[121], copie[60];
    int   i,j;
    char  systeme,bravais,*pos;
    int   holo,centro,nops;
    int   size = 12;
    float  *l_coef;
    SYMGRP *l_sym;

    purge_cr();
    fscanf(file," %s", name);
    fgets (ligne, 120, file);  /* the end of the line with comments... */
    if (feof(file)) return NULL;
    push_err_msg("read_sym (xnd_rsym.c)");
    pos=ligne;
    while ( *pos == ' ' || *pos == '\r' ) pos++;
    purge_cr();
    fscanf(file," %[CQHTROMNLA*]", copie);
    systeme = copie[0];
    purge_cr();
    i = fscanf(file," %d%d%d%*[\r]", &holo, &centro, &nops);
    if (feof(file)) { pop_err_msg(); return NULL; }
    tst_abort(( i<3  || nops > 1000), "invalid data in symgrp header '%s'", name);
    if (strlen(name)>=SYM_ID) ligne[SYM_ID-1]=0;
    bravais = name[0];
    strup(name);
    if (bravais == '*')
    if ( systeme == '*') size = holo*(holo+1);
      else tst_abort (1,"Complement de sym non valide");
    l_sym = (SYMGRP *)tst_alloc(1,sizeof(SYMGRP)+(size*nops-1)*sizeof(float), "symgrp");
    n_sym++;
    strcpy(l_sym->nom, name);
    strncpy(l_sym->comment, pos, NAME_LG-1);
    l_sym->systeme=systeme;
    l_sym->holoedre=holo;
    l_sym->bravais=bravais;
    l_sym->centro=centro;
    l_sym->nops=nops;
    l_sym->size=size;
    for (j=0,l_coef=l_sym->coef;j<nops*size;j++,l_coef++) *l_coef = 0;
    for (i=0,l_coef=l_sym->coef;i<nops;i++,l_coef+=size)
    {   fscanf(file," %[^;\n]%*[;\n]",ligne);
	while(!analyse_xyz_pos (ligne, l_coef, l_sym));
	};
   pop_err_msg();
   return(l_sym);
   }

/* ------------------------------------------------------------------ */

SYMGRP *get_sym (P_(char *) modele)
#ifdef NO_PROTO
char *modele;
#endif
{   SYMGRP *l_sym;
    FILE   *sym_fich;
    int i;

    push_err_msg("get_sym(xnd_rsym.c)");
    strup (modele);
    for (i=0;i<n_sym;i++)
	 if (strcmp(lect->symgrp[i]->nom,modele)==0) 
	 {   pop_err_msg();
	     return(lect->symgrp[i]);
	     }
    sym_fich = tst_open (lect->sym_name,"r","Open sym file");
    do
    {   l_sym=read_sym(sym_fich);
	 if (l_sym && strcmp(l_sym->nom,modele)==0)
	 {   if (lect->prt_sym>1) print_sym (l_sym, 0);
	     switch (l_sym->systeme)
	     {   case 'C': case 'Q': case 'H': case 'R': case 'T':
		 case 'O': case 'M': case 'N': case 'A': break;
		 default : if (lect->prt_sym<=1) print_sym (l_sym, 0);
			   termine (10,"Invalid Lattice");
		 };
	     fclose (sym_fich);
	     lect->symgrp[n_sym-1]=l_sym;
             pop_err_msg();
	     return(l_sym);
	     };
	 if (l_sym) free(l_sym);
	 n_sym--;
	 }
    while (!feof(sym_fich));
    tst_abort (1,"No SYM : %s",modele);
    pop_err_msg();
    return(NULL);
    }
